#include "decode_rtp.h"
#include "decode_ip.h"
#include "../packets/packet.h"
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

//#define DEBUG
//#define DEBUGPATH
//#define DEBUGPACKET

extern GlobalVars	Globals;

int	RTPDecoderID;
int	IPDecoderID;

/*********************************************
* Parse SIP Message 
**********************************************/

int RTPHeaderVerification(RTPData* data){
	if(data->Header->version!=2) return FALSE;
	if((data->Header->pt>=20)&&(data->Header->pt<=24)) return FALSE;
	if(data->Header->pt==27) return FALSE;
	if(data->Header->pt==29) return FALSE;
	if(data->Header->pt==30) return FALSE;
	if((data->Header->pt>=35)&&(data->Header->pt<=71)) return FALSE;
	if((data->Header->pt>=77)&&(data->Header->pt<=95)) return FALSE;
	return TRUE;
}

/***************************************
* Apply the rtp decoding
****************************************/
void* DecodeRTP(int PacketSlot){
	RTPData*		data;
    IPData*         ip_data;
	PacketRec*		p;
	unsigned int	saddr;
	unsigned int	daddr;
//	osip_message_t*		data;
	
#ifdef DEBUGPATH
	printf("In DecodeRTP\n");
#endif

#ifdef DEBUG
	printf("Decoding RTP Header\n");
#endif

	p=&Globals.Packets[PacketSlot];

    if (!GetDataByID(PacketSlot, IPDecoderID, (void**)&ip_data)){
        printf("Failed to get IP header data\n");
        return NULL;
    }

	saddr=ip_data->Header->saddr;
	daddr=ip_data->Header->daddr;

#ifdef DEBUG
	printf("IP %s->",inet_ntoa(*(struct in_addr*)&saddr));
	printf("%s\n",inet_ntoa(*(struct in_addr*)&daddr));
#endif

	data=malloc(sizeof(RTPData));
	data->Header=(RTPHdr*)(p->RawPacket+p->BeginData);
	if (!RTPHeaderVerification(data)) {
#ifdef DEBUG
		printf("Failed to parse RTP Message\n");
#endif
		free(data);
		return FALSE;
	}

#ifdef DEBUGPACKET
	printf("Protocol Version=%i\n",data->Header->version);
	printf("Padding=%i\n",data->Header->p);
	printf("Extension=%i\n",data->Header->x);
	printf("CC=%i\n",data->Header->cc);
	printf("Marker=%i\n",data->Header->m);
	printf("Payload Type=%i\n",data->Header->pt);
	printf("Seq=%i\n",data->Header->seq);
	printf("Timestamp=%i\n",data->Header->timestamp);
	printf("SSRC=%i\n",data->Header->ssrc);
//	printf("CSRC=%i\n",data->Header->csrc[1]);
#endif

	return data;
}

/*************************************
* Set up the decoder
*************************************/
int InitDecoderRTP(){
	int DecoderID;

#ifdef DEBUGPATH
	printf("In InitDecoderRTP\n");
#endif
	
	if ((DecoderID=CreateDecoder("RTP"))==DECODER_NONE){
#ifdef DEBUG
		printf("Couldn't Allocate RTP Decoder\n");
#endif	
		return FALSE;
	}
	
	Globals.Decoders[DecoderID].DecodeFunc=DecodeRTP;
	if (!DecoderAddDecoder(GetDecoderByName("UDP"), DecoderID)){
		printf("Failed to Bind RTP Decoder to UDP Decoder\n");
		return FALSE;
	}

	RTPDecoderID=GetDecoderByName("RTP");
	IPDecoderID=GetDecoderByName("IP");

	return TRUE;
}
