#include "decode_sip.h"
#include "../packets/packet.h"
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <osipparser2/osip_parser.h>
#include <osipparser2/sdp_message.h>
#include <osipparser2/osip_list.h>

//#define DEBUG
//#define DEBUGPATH
//#define DEBUGPACKET
//#define DEBUGPARSING

extern GlobalVars	Globals;

int	SIPDecoderID;

/*********************************************
* Parse SIP Message 
**********************************************/
int SIPParser(char* message,osip_message_t *sip){
	char* 		result;
	size_t 		length;
#ifdef DEBUGPATH
    printf("In SIPParser\n");
#endif  
	parser_init();
	osip_message_init(&sip);
	if (osip_message_parse(sip,message,strlen(message))!=0){
#ifdef DEBUGPARSING
		printf ("ERROR: failed while parsing SIP MESSAGE!\n");
#endif
		osip_message_free(sip);
		return FALSE;
	}
#ifdef DEBUGPACKET
	if (osip_message_to_str(sip,&result,&length)!=-1){
		fwrite(result,1,length,stdout);
	}
#endif
	osip_message_free(sip);
//	return sip;
	return TRUE;
}

osip_message_t* SIPParser2(char* message,osip_message_t *sip){
	char* 		result;
	size_t 		length;
#ifdef DEBUGPATH
    printf("In SIPParser2\n");
#endif  
	parser_init();
#ifdef DEBUGPATH
    printf("In SIPParser2 - 1\n");
#endif  
	osip_message_init(&sip);
#ifdef DEBUGPATH
    printf("In SIPParser2 - 2\n");
#endif  
	if (osip_message_parse(sip,message,strlen(message))!=0){
#ifdef DEBUGPARSING
		printf ("ERROR: failed while parsing SIP MESSAGE!\n");
#endif
		osip_message_free(sip);
		return NULL;
	}
#ifdef DEBUGPATH
    printf("In SIPParser2 - 3\n");
#endif  
#ifdef DEBUGPACKET
	if (osip_message_to_str(sip,&result,&length)!=-1){
		fwrite(result,1,length,stdout);
	}
#endif
#ifdef DEBUGPATH
    printf("In SIPParser2 - 3\n");
#endif  
	if (sip!=NULL) return sip;
	return FALSE;
}
/*********************************************
* Parse SDP Message 
**********************************************/
sdp_message_t* SDPParser(osip_message_t *sip){
	sdp_message_t *sdp;
	osip_body_t *oldbody;
	int pos;
	int i;
#ifdef DEBUGPATH
    printf("In SDPParser\n");
#endif  

	pos = 0;
	while (!osip_list_eol(&sip->bodies, pos)){
		oldbody = (osip_body_t *) osip_list_get (&sip->bodies, pos);
		pos++;
		sdp_message_init(&sdp);
		if (sdp_message_parse(sdp,oldbody->body)!=0){
#ifdef DEBUG
			printf("ERROR: Bad SDP!\n");
			sdp_message_free (sdp);
			return NULL;
#endif
		}
	}
	return sdp;
}

/*sdp_media_t* MEDIAParser(sdp_message_t *sdp){ //sip
	sdp_media_t *media; //sdp
	sdp_media_t *oldmedia;
	int pos;
	int i;
#ifdef DEBUGPATH
    printf("In MEDIAParser\n");
#endif  

	pos = 0;
	while (!osip_list_eol(&sdp->m_medias, pos)){
		oldmedia = (sdp_media_t *) osip_list_get (&sdp->m_medias, pos);
		pos++;
		sdp_media_init(&media);
		if (sdp_message_parse(sdp,oldbody->body)!=0){
#ifdef DEBUG
			printf("ERROR: Bad SDP!\n");
			sdp_message_free (sdp);
			return NULL;
#endif
		}
	}
	return media;
}*/

/***************************************
* Apply the sip decoding
****************************************/
void* DecodeSIP(int PacketSlot){
	PacketRec*			p;
	osip_message_t*		data;
	int			ret;
	
#ifdef DEBUGPATH
	printf("In DecodeSIP\n");
#endif

#ifdef DEBUG
	printf("Decoding SIP Header\n");
#endif

	p=&Globals.Packets[PacketSlot];

	data=malloc(sizeof(osip_message_t));
//	data=SIPParser((char*)(p->RawPacket+p->BeginData),data);
	ret=SIPParser((char*)(p->RawPacket+p->BeginData),data);
	if (ret==FALSE) {
		free(data);
#ifdef DEBUG
		printf("Failed to parse SIP Message\n");
#endif
		return FALSE;
	}

/*#ifdef DEBUG
	printf("VERSION = %s\n",data->sip_version);
	printf("METHOD = %s\n",data->sip_method);
	printf("STATUSCODE = %i\n",data->status_code);
	printf("REASONPHRASE = %s\n",data->reason_phrase);
#endif*/

	return data;
}

/*************************************
* Set up the decoder
*************************************/
int InitDecoderSIP(){
	int DecoderID;

#ifdef DEBUGPATH
	printf("In InitDecoderSIP\n");
#endif
	
	if ((DecoderID=CreateDecoder("SIP"))==DECODER_NONE){
#ifdef DEBUG
		printf("Couldn't Allocate SIP Decoder\n");
#endif	
		return FALSE;
	}
	
	Globals.Decoders[DecoderID].DecodeFunc=DecodeSIP;
	if (!DecoderAddDecoder(GetDecoderByName("IP"), DecoderID)){
		printf("Failed to Bind SIP UDP Decoder to IP Decoder\n");
		return FALSE;
	}

	SIPDecoderID=GetDecoderByName("SIP");

	return TRUE;
}
