#include "test_rtp_flood.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../decoders/decode_rtp.h"
#include "../decoders/decode_udp.h"
#include "../decoders/decode_ip.h"
#include "../packets/packet.h"
#include <arpa/inet.h>
#include "../engine/num_list.h"
#include "../engine/bits.h"

extern GlobalVars	Globals;

typedef struct my_rtp_flood_data{
	unsigned short					source;
	unsigned short					dest;
	unsigned int					saddr;
	unsigned int					daddr;
	int								count;
	int								ttl;
	struct my_rtp_flood_data*		Next;
} RTPFloodData;

//#define DEBUG
//#define DEBUGMATCH
//#define DEBUGPATH

int TTLRTPFlood;
int ThresholdRTPFlood;
int RTPDecoderID;
int IPDecoderID;
int UDPDecoderID;
RTPFloodData* RTPFloodDataHead;

int RTPFloodCount(unsigned int saddr, unsigned int daddr, unsigned short source, unsigned short dest){
	RTPFloodData*		t;
	RTPFloodData*		last;

	if (saddr==0) return 0;
	if (daddr==0) return 0;

	t=RTPFloodDataHead;
	last=t;
	while(t){
		if ((t->saddr==saddr)&&(t->daddr==daddr)&&(t->source==source)&&(t->dest==dest)){
			return t->count;
		}
		last=t;
		t=t->Next;
	}
	return 0;
}

/******************************************
* Apply the Test with collapsed rules
******************************************/
int TestRTPFlood(int PacketSlot, TestNode* Nodes){
	int					difference;
	RTPFloodData*		data;
	RTPFloodData*		t;
	RTPFloodData*		last;
	TestNode*			Node;
	IPData*				IData;
	RTPData*			RData;
	UDPData*			UData;
	int					i;
	PacketRec*			p;

#ifdef DEBUGPATH
	printf("In TestRTPFlood\n");
#endif

#ifdef DEBUG
	printf("Testing RTP Flood\n");
#endif	
	
	IData=NULL;
	RData=NULL;
	UData=NULL;

	if (!Nodes) return FALSE;
	
	p=&Globals.Packets[PacketSlot];
	
	/*get the sip method of the packet*/
	if (!GetDataByID(PacketSlot, IPDecoderID, (void**)&IData)){
#ifdef DEBUG
		printf("Failed to get RTP Flood IP data\n");
#endif	
	}
	if (!GetDataByID(PacketSlot, UDPDecoderID, (void**)&UData)){
#ifdef DEBUG
		printf("Failed to get RTP Flood UDP data\n");
#endif	
	}

	if (GetDataByID(PacketSlot, RTPDecoderID, (void**)&RData)){
		if (!RTPHeaderVerification(RData)) {
#ifdef DEBUG
			printf("Failed to get SIP RTP Bye Attack RTP data\n");
#endif	
			RData=NULL;
		}
	}


#ifdef DEBUGMATCH
	printf("**************************************\n");
	printf("Before applying sip rtp bye tests\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	



    Node=Nodes;
 	while(Node){
       	if (RuleIsActive(PacketSlot,Node->RuleID)){
			if ((IData)&&(RData)&&(UData)){
				if (RTPHeaderVerification(RData)){
#ifdef DEBUG
					printf("         RTP DATA\n");
					printf("SRC_RTP %s\n",inet_ntoa(*(struct in_addr*)&IData->Header->saddr));
					printf("DST_RTP %s\n",inet_ntoa(*(struct in_addr*)&IData->Header->daddr));
    				printf("USRC %u\n", ntohs(UData->Header->source));
    				printf("UDST %u\n", ntohs(UData->Header->dest));
#endif
					data=calloc(sizeof(RTPFloodData),1);
					data->saddr=IData->Header->saddr;
					data->daddr=IData->Header->daddr;
					data->source=UData->Header->source;
					data->dest=UData->Header->dest;

					if (!RTPFloodDataHead) {
#ifdef DEBUG
						printf("First RTP Flood Data\n");
#endif	
						data->count=0;
						data->ttl=0;
#ifdef DEBUG
						printf("COUNT=%i\n",data->count);
#endif	
						RTPFloodDataHead=data;
					} else {
#ifdef DEBUG
						printf("Adding RTP Flood Data\n");
#endif	
						t=RTPFloodDataHead;
						last=t;
						i=0;
						while(t){
							if ((data->saddr==t->saddr)&&(data->daddr==t->daddr)&&(data->source==t->source)&&(data->dest==t->dest)){
								i=i+1;
								t->ttl=0;
								t->count=t->count + 1;
							}else{
								t->ttl=t->ttl+1;
							}
							if (t->ttl>TTLRTPFlood){
#ifdef DEBUG
								printf("--------------TTL over\n");
#endif	
								if (last==t) {
									if (t->Next!=NULL){
										RTPFloodDataHead=t->Next;
										t=RTPFloodDataHead;
										last=t;
									} else {
										RTPFloodDataHead=NULL;
										t=NULL;
									}
								} else if (t->Next==NULL) {
									last->Next=NULL;
									t=NULL;
								} else {
									last->Next=t->Next;
									t=t->Next;
								}
							} else {
								last=t;
								t=t->Next;
							}
						}
						if (i==0) { 
#ifdef DEBUG
							printf("This is a new one\n");
#endif		
							data->count=0;
							data->ttl=0;
							last->Next=data;
						}
					}

#ifdef DEBUG
					printf("Looking to match\n");
#endif		
					if (!IData->Header->saddr) IData->Header->saddr=0;
					if (!IData->Header->daddr) IData->Header->daddr=0;
					difference = RTPFloodCount(IData->Header->saddr,IData->Header->daddr,UData->Header->source,UData->Header->dest)-RTPFloodCount(IData->Header->daddr,IData->Header->saddr,UData->Header->dest,UData->Header->source);

					if (difference<0) difference*=-1;
#ifdef DEBUG
					printf("DIFFERENCE === %i\n",difference);
#endif
					if (difference<ThresholdRTPFlood){
#ifdef DEBUGMATCH
        		   	    printf("RTP Flood doesn't match, Rule = %i\n",Node->RuleID);
#endif
   		       			SetRuleInactive(PacketSlot,Node->RuleID);
					}
				}else{
       				SetRuleInactive(PacketSlot,Node->RuleID);
				}
				data=NULL;
				free(data);
			}else{
       			SetRuleInactive(PacketSlot,Node->RuleID);
			}
		}
       	Node=Node->Next;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	
	return TRUE;
}

/******************************************
* Add a rule node to this test
******************************************/
int RTPFloodAddNode(int TestID, int RuleID, char* Args){
	RTPData*			data;

#ifdef DEBUGPATH
	printf("In RTPFloodAddNode\n");
#endif

#ifdef DEBUG
	printf("Adding a Node with args %s\n",Args);
#endif

	ThresholdRTPFlood = atoi(Args);
	TTLRTPFlood=ThresholdRTPFlood * 2;
	data=NULL;
	data=calloc(sizeof(RTPData),1);
	
#ifdef DEBUG
	printf("First RTP Flood\n");
#endif	
	TestAddNode(TestID, RuleID, (void*)data);
	data=NULL;
	free(data);

	return TRUE;
}

/****************************************
* Set up the test of the SIP RTP Bye Attack Field
*****************************************/
int InitTestRTPFlood(){
	int	TestID;

#ifdef DEBUGPATH
	printf("In InitTestRTPFlood\n");
#endif

	RTPFloodDataHead=NULL;

	TestID=CreateTest("RTPFlood");
	if (TestID==TEST_NONE) return FALSE;
	
	if (!BindTestToDecoder(TestID, "RTP")){
		printf("Failed to Bind to RTP\n");
		return FALSE;
	} 
	
	snprintf(Globals.Tests[TestID].ShortName, MAX_NAME_LEN, "rtp_flood");
	Globals.Tests[TestID].AddNode=RTPFloodAddNode;
	Globals.Tests[TestID].TestFunc=TestRTPFlood;
	
	RTPDecoderID=GetDecoderByName("RTP");
	IPDecoderID=GetDecoderByName("IP");
	UDPDecoderID=GetDecoderByName("UDP");

	return TRUE;
}
