#include "test_sip_dictionary_attack.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include "../decoders/decode_sip.h"
#include "../decoders/decode_ip.h"
#include "../packets/packet.h"
#include <arpa/inet.h>
//#include "../engine/num_list.h"
//#include "../engine/bits.h"
#include <osipparser2/osip_parser.h>

extern GlobalVars	Globals;

typedef struct my_sip_dictionary_attack_data{
	unsigned int				saddr;
	unsigned int				daddr;
	char*					username;
	unsigned int				count;
	unsigned int				numregister;
	unsigned int				trigger:1;
	unsigned int				timeout;
	struct my_sip_dictionary_attack_data*	Next;
} SIPDictionaryAttackData;

//#define DEBUG
//#define DEBUGMATCH
//#define DEBUGPATH

int IPDecoderID;
int SIPDecoderID;
SIPDictionaryAttackData* SIPDictionaryAttackDataHead;
int slice;
int timeout;
unsigned int numregister;
pthread_mutex_t			DictionaryAttackMutex;
int				DictionaryAttackLockID;

int DictionaryAttackTimerFunc(int TimerID, int Time, void* User){
	struct timeval ttime;
	SIPDictionaryAttackData*			t;
	SIPDictionaryAttackData*			last;
	int i;

#ifdef DEBUGPATH
	printf("In DictionaryAttackTimerFunc\n");
#endif
	t=SIPDictionaryAttackDataHead;
	last=t;
/*	i=0;*/
//	printf("!\n");
	while(t){
//		printf(".\n");
		if(t->count>0) {
			t->count=0;
			t->numregister+=1;
			if (t->numregister>numregister) {
				t->trigger=1;
			}
		} else {
			t->timeout+=1;
			t->numregister=0;
		}
		if(t->timeout>timeout){
			if (last==t) {
				if (t->Next!=NULL){
					//free(SIPDictionaryAttackDataHead);
					SIPDictionaryAttackDataHead=t->Next;
					t=SIPDictionaryAttackDataHead;
					last=t;
				} else {
					//free(SIPDictionaryAttackDataHead);
					SIPDictionaryAttackDataHead=NULL;
					t=NULL;
				}
			} else if (t->Next==NULL) {
				//free(t);
				last->Next=NULL;
				t=NULL;
			} else {
				//free(last->Next);
				last->Next=t->Next;
				t=t->Next;
			}
		}
		last=t;
		t=t->Next;
	}
	printf("DEFAULT MESSAGE: Dictionary Attack Timer\n");
}

/******************************************
* Apply the Test with collapsed rules
******************************************/
int TestSIPDictionaryAttack(int PacketSlot, TestNode* Nodes){
	int			 					Action;
	SIPDictionaryAttackData*			data;
	SIPDictionaryAttackData*			t;
	SIPDictionaryAttackData*			last;
	TestNode*						Node;
	osip_message_t*					SData;
	IPData*							IData;
	int								i;
	PacketRec*						p;

#ifdef DEBUGPATH
	printf("In TestSIPDictionaryAttack\n");
#endif

	IData=NULL;
	SData=NULL;

	if (!Nodes) return FALSE;
	
	p=&Globals.Packets[PacketSlot];
	
	/*get the sip method of the packet*/
	if (!GetDataByID(PacketSlot, IPDecoderID, (void**)&IData)){
#ifdef DEBUG
		printf("Failed to get SIP Invite Flood IP data\n");
#endif	
	}
	SData=SIPParser2((char*)(p->RawPacket+p->BeginData),SData);
	if (SData==NULL){	
		osip_message_free(SData);
#ifdef DEBUG
		printf("Failed to get SIP Method data\n");
#endif 
		return FALSE;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	printf("Before applying sip register flood tests\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	

    Node=Nodes;
	while(Node){
	       	if (RuleIsActive(PacketSlot,Node->RuleID)){
			if (SData){
#ifdef DEBUG
				printf("         SIP DATA\n");
#endif	
				Action=0;
				if ((SData->sip_method!=NULL)&&(strcmp(SData->sip_method,"REGISTER")==0)) Action = 1;
				if (SData->status_code == 401) Action = 1;
				if (SData->status_code == 403) Action = 1;
				if (Action!=0) {
#ifdef DEBUG
					printf("         	DICTIONARY ATTACK DATA\n");
#endif	
					data=calloc(sizeof(SIPDictionaryAttackData),1);
					data->saddr=IData->Header->saddr;
					data->daddr=IData->Header->daddr;
					
					//dchar *s1 = "this string";
					char *t1 = SData->from->url->username;
					//char *s2;
					char *t2;
					if (data->username = malloc(strlen(SData->from->url->username) + 1)) {
						t2 = data->username;
						while ((*t2++ = *t1++));
						if (!SIPDictionaryAttackDataHead) {
#ifdef DEBUG
							printf(">>>> First Dictionary Attack Data\n");
							printf("IP %s->",inet_ntoa(*(struct in_addr*)&data->saddr));
							printf("%s\n",inet_ntoa(*(struct in_addr*)&data->daddr));
							printf("USERNAME %s\n",data->username);
#endif	
							data->timeout=0;
							data->count=0;
							data->trigger=0;
							SIPDictionaryAttackDataHead=data;
      							SetRuleInactive(PacketSlot,Node->RuleID);
						} else {
							t=SIPDictionaryAttackDataHead;
							last=t;
							i=0;
							while(t){
#ifdef DEBUG
									printf(">>>> Adding Dictionary Attack Data\n");
									printf("IP %s->",inet_ntoa(*(struct in_addr*)&data->saddr));
									printf("%s\n",inet_ntoa(*(struct in_addr*)&data->daddr));
									printf("USERNAME %s\n",data->username);
									printf("IP %s->",inet_ntoa(*(struct in_addr*)&t->saddr));
									printf("%s\n",inet_ntoa(*(struct in_addr*)&t->daddr));
									printf("USERNAME %s\n",t->username);
#endif
								if ((data->saddr==t->saddr)&&(data->daddr==t->daddr)&&(strcmp(data->username,t->username)==0)||(data->saddr==t->daddr)&&(data->daddr==t->saddr)&&(strcmp(data->username,t->username)==0)){
									i=i+1;
									if (t->trigger==0) {
      										SetRuleInactive(PacketSlot,Node->RuleID);
									} else {
										t->trigger=0;
									}
									t->timeout=0;
									t->count+=1;
								}
								last=t;
								t=t->Next;
							}
							if (i==0) { 
#ifdef DEBUG
								printf(">>>> This is a new one\n");
								printf("IP %s->",inet_ntoa(*(struct in_addr*)&data->saddr));
								printf("%s\n",inet_ntoa(*(struct in_addr*)&data->daddr));
								printf("USERNAME %s\n",data->username);
#endif
								data->timeout=0;
								data->count=0;
								data->trigger=0;
								last->Next=data;
      								SetRuleInactive(PacketSlot,Node->RuleID);
							}
						}
					} else {
   		       				SetRuleInactive(PacketSlot,Node->RuleID);
					}
					//free(data);
				} else {
   		       			SetRuleInactive(PacketSlot,Node->RuleID);
				}
			}else{
       				SetRuleInactive(PacketSlot,Node->RuleID);
			}
		}
       		Node=Node->Next;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif
	if (SData!=NULL) osip_message_free(SData);
	return TRUE;
}

/******************************************
* Add a rule node to this test
******************************************/
int SIPDictionaryAttackAddNode(int TestID, int RuleID, char* Args){
	SIPDictionaryAttackData*			data;

#ifdef DEBUGPATH
	printf("In SIPDictionaryAttackAddNode\n");
#endif

	sscanf(Args, "%i,%i,%i",&slice,&timeout,&numregister);
#ifdef DEBUGPATH
	printf("SLICE=%i\n",slice);
#endif
	data=calloc(sizeof(SIPDictionaryAttackData),1);
	
	CreateTimer("Dictionary Attack Testing", slice, DictionaryAttackTimerFunc, (void*)NULL);	
	TestAddNode(TestID, RuleID, (void*)data);
	
	//free(data);

	return TRUE;
}

/****************************************
* Set up the test of the SIP Dictionary Attack Field
*****************************************/
int InitTestSIPDictionaryAttack(){
	int	TestID;

#ifdef DEBUGPATH
	printf("In InitTestSIPDictionaryAttack\n");
#endif

	SIPDictionaryAttackDataHead=NULL;

	TestID=CreateTest("SIPDictionaryAttack");
	if (TestID==TEST_NONE) return FALSE;
	
	if (!BindTestToDecoder(TestID, "SIP")){
#ifdef DEBUG
		printf("Failed to Bind to SIP\n");
#endif
		return FALSE;
	} 
	
	snprintf(Globals.Tests[TestID].ShortName, MAX_NAME_LEN, "dictionary_attack");
	Globals.Tests[TestID].AddNode=SIPDictionaryAttackAddNode;
	Globals.Tests[TestID].TestFunc=TestSIPDictionaryAttack;
	
	IPDecoderID=GetDecoderByName("IP");
	SIPDecoderID=GetDecoderByName("SIP");

	return TRUE;
}

