#include "test_sip_method.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../decoders/decode_sip.h"
#include "../packets/packet.h"
#include <arpa/inet.h>
#include "../engine/num_list.h"
#include "../engine/bits.h"
#include <osipparser2/osip_parser.h>

extern GlobalVars	Globals;

typedef struct my_sip_data{
	char*					Method;
	struct my_sip_data*		Next;
} SIPMethod;

//#define DEBUG
//#define DEBUGMATCH
//#define DEBUGPATH

int SIPDecoderID;
SIPMethod* SIPMethodHead;
char ListMethod[1024];


/******************************************
* Apply the Test with collapsed rules
******************************************/
int TestSIPMethod(int PacketSlot, TestNode* Nodes){
	unsigned char* 		Method;
	SIPMethod*			t;
	SIPMethod*			last;
	TestNode*			Node;
	osip_message_t*		SData;
	int					i;
	PacketRec*			p;
	int					hab;
	char				tMethod[10];

#ifdef DEBUGPATH
	printf("In TestSIPMethod\n");
#endif

#ifdef DEBUG
	printf("Testing SIP Method\n");
#endif	
	
	if (!Nodes) return FALSE;
	
	p=&Globals.Packets[PacketSlot];
	
	/*get the sip method of the packet*/
	SData=SIPParser2((char*)(p->RawPacket+p->BeginData),SData);
	if (SData==NULL){	
		osip_message_free(SData);
#ifdef DEBUG
		printf("Failed to get SIP Method data\n");
#endif 
		return FALSE;
	}

	Method=SData->sip_method;

#ifdef DEBUG
	printf("%s\n",Method);
#endif
	
#ifdef DEBUGMATCH
	printf("**************************************\n");
	printf("Before applying sip method tests\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	

	
	Node=Nodes;
	while(Node){
		if (RuleIsActive(PacketSlot,Node->RuleID)){
			if((Method!=NULL)&&(strlen(Method)<10)){
				for (i=0;i<10;i++)tMethod[i]='\0';
				tMethod[0]=',';
				for (i=0;i<strlen(Method);i++) tMethod[i+1]=Method[i];
				tMethod[strlen(Method)+1]=',';
				if(strstr(ListMethod,tMethod)==NULL) {
               				SetRuleInactive(PacketSlot,Node->RuleID);
				}
			}else{
		               	SetRuleInactive(PacketSlot,Node->RuleID);
			}
		}
		Node=Node->Next;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif
		
	if (SData!=NULL) osip_message_free(SData);
	return TRUE;
}

/******************************************
* Add a rule node to this test
******************************************/
int SIPMethodAddNode(int TestID, int RuleID, char* Args){
	SIPMethod*			data;
	int					i;
	char*				method;
	char*				m;
	char*				tmp;
	char				listmethod[1024];
	char				tArgs[1024];

#ifdef DEBUGPATH
	printf("In SIPMethodAddNode\n");
#endif

#ifdef DEBUG
	printf("Adding a Node with args %s\n",Args);
#endif

	for (i=0;i<1024;i++) listmethod[i]='\0';
	listmethod[0]=',';
	for (i=0;i<strlen(Args);i++) listmethod[i+1]=Args[i];
	listmethod[strlen(Args)+1]=',';
	strcpy(ListMethod,listmethod);
	TestAddNode(TestID, RuleID, (void*)NULL);
	return TRUE;
}

/****************************************
* Set up the test of the SIP Method Field
*****************************************/
int InitTestSIPMethod(){
	int	TestID;

#ifdef DEBUGPATH
	printf("In InitTestSIPMethod\n");
#endif

	SIPMethodHead=NULL;

	TestID=CreateTest("SIPMethod");
	if (TestID==TEST_NONE) return FALSE;
	
	if (!BindTestToDecoder(TestID, "SIP")){
#ifdef DEBUG
		printf("Failed to Bind to SIP\n");
#endif
		return FALSE;
	} 
	
	snprintf(Globals.Tests[TestID].ShortName, MAX_NAME_LEN, "method");
	Globals.Tests[TestID].AddNode=SIPMethodAddNode;
	Globals.Tests[TestID].TestFunc=TestSIPMethod;
	
	SIPDecoderID=GetDecoderByName("SIP");

	return TRUE;
}
