#include "test_sip_not_method.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../decoders/decode_sip.h"
#include "../packets/packet.h"
#include <arpa/inet.h>
#include "../engine/num_list.h"
#include "../engine/bits.h"
#include <osipparser2/osip_parser.h>

extern GlobalVars	Globals;

typedef struct my_sip_data{
	char*					Method;
	struct my_sip_data*		Next;
} SIPNotMethod;

//#define DEBUG
//#define DEBUGMATCH
//#define DEBUGPATH

int SIPDecoderID;
SIPNotMethod* SIPNotMethodHead;
char ListNotMethod[1024];


/******************************************
* Apply the Test with collapsed rules
******************************************/
int TestSIPNotMethod(int PacketSlot, TestNode* Nodes){
	unsigned char* 		NotMethod;
	SIPNotMethod*		t;
	SIPNotMethod*		last;
	TestNode*			Node;
	osip_message_t*		SData;
	int					i;
	PacketRec*			p;
	int					hab;
	char				tNotMethod[12];
	int				Status;

#ifdef DEBUGPATH
	printf("In TestSIPNotMethod\n");
#endif

#ifdef DEBUG
	printf("Testing SIP Not Method\n");
#endif	
	
	if (!Nodes) return FALSE;
	
	p=&Globals.Packets[PacketSlot];
	
	/*get the sip method of the packet*/
	//printf("ERRO 0\n");
	SData=SIPParser2((char*)(p->RawPacket+p->BeginData),SData);
	//printf("ERRO 1\n");
	if (SData==NULL){	
		osip_message_free(SData);
#ifdef DEBUG
		printf("Failed to get SIP Method data\n");
#endif 
		return FALSE;
	}

	//printf("ERRO 2\n");
	NotMethod=SData->sip_method;
	Status=SData->status_code;
	//printf("ERRO 3\n");
#ifdef DEBUGMATCH
	printf("**************************************\n");
	printf("Before applying sip method tests\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif

	
	Node=Nodes;
	while(Node){
		if (RuleIsActive(PacketSlot,Node->RuleID)){
#ifdef DEBUG
			if (NotMethod==NULL) printf("METHOD NULL\n");
#endif
			if((NotMethod!=NULL)&&(strlen(NotMethod)<10)){
				for (i=0;i<12;i++) tNotMethod[i]='\0';
				tNotMethod[0]=',';
				for (i=0;i<strlen(NotMethod);i++) tNotMethod[i+1]=NotMethod[i];
				tNotMethod[strlen(NotMethod)+1]=',';
#ifdef DEBUG
				printf("%s\n%s\n",tNotMethod,ListNotMethod);
#endif
				if(strstr(ListNotMethod,tNotMethod)!=NULL) {
		               		SetRuleInactive(PacketSlot,Node->RuleID);
				}
			} else 
			if (Status>0) {
#ifdef DEBUG
				printf("STATUS CODE = %i\n",Status);
#endif
		              	SetRuleInactive(PacketSlot,Node->RuleID);
			}
		}else{
               		SetRuleInactive(PacketSlot,Node->RuleID);
		}
       		Node=Node->Next;
	}

	//printf("ERRO 4\n");
#ifdef DEBUGMATCH
	printf("**************************************\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif
		
	//printf("ERRO 5\n");
	if (SData != NULL) osip_message_free(SData);
	//printf("ERRO 6\n");
	return TRUE;
}

/******************************************
* Add a rule node to this test
******************************************/
int SIPNotMethodAddNode(int TestID, int RuleID, char* Args){
	int					i;
	char*				notmethod;
	char*				m;
	char*				tmp;
	char				listnotmethod[1024];
	char				tArgs[1024];

#ifdef DEBUGPATH
	printf("In SIPNotMethodAddNode\n");
#endif

#ifdef DEBUG
	printf("Adding a Node with args %s\n",Args);
#endif

	for (i=0;i<1024;i++) listnotmethod[i]='\0';
	listnotmethod[0]=',';
	for (i=0;i<strlen(Args);i++) listnotmethod[i+1]=Args[i];
	listnotmethod[strlen(Args)+1]=',';
	strcpy(ListNotMethod,listnotmethod);
	TestAddNode(TestID, RuleID, (void*)NULL);

	return TRUE;
}

/****************************************
* Set up the test of the SIP Not Method Field
*****************************************/
int InitTestSIPNotMethod(){
	int	TestID;

#ifdef DEBUGPATH
	printf("In InitTestSIPNotMethod\n");
#endif

	SIPNotMethodHead=NULL;

	TestID=CreateTest("SIPNotMethod");
	if (TestID==TEST_NONE) return FALSE;
	
	if (!BindTestToDecoder(TestID, "SIP")){
#ifdef DEBUG
		printf("Failed to Bind to SIP\n");
#endif
		return FALSE;
	} 
	
	snprintf(Globals.Tests[TestID].ShortName, MAX_NAME_LEN, "not_method");
	Globals.Tests[TestID].AddNode=SIPNotMethodAddNode;
	Globals.Tests[TestID].TestFunc=TestSIPNotMethod;
	
	SIPDecoderID=GetDecoderByName("SIP");

	return TRUE;
}
