#include "test_sip_not_version.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../decoders/decode_sip.h"
#include "../packets/packet.h"
#include <arpa/inet.h>
#include "../engine/num_list.h"
#include "../engine/bits.h"
#include <osipparser2/osip_parser.h>

extern GlobalVars	Globals;

typedef struct my_sip_data{
	char*					Version;
	struct my_sip_data*		Next;
} SIPVersion;

//#define DEBUG
//#define DEBUGMATCH
//#define DEBUGPATH

int SIPDecoderID;
SIPVersion* SIPVersionHead;
char ListVersion[1024];


/******************************************
* Apply the Test with collapsed rules
******************************************/
int TestSIPNotVersion(int PacketSlot, TestNode* Nodes){
	unsigned char* 		Version;
	SIPVersion*			t;
	SIPVersion*			last;
	TestNode*			Node;
	osip_message_t*		SData;
	int					i;
	PacketRec*			p;
	int					hab;
	char				tVersion[12];

#ifdef DEBUGPATH
	printf("In TestSIPNotVersion\n");
#endif

#ifdef DEBUG
	printf("Testing SIP Not Version\n");
#endif	
	
	if (!Nodes) return FALSE;
	
	p=&Globals.Packets[PacketSlot];
	
	/*get the sip version of the packet*/
	if (!GetDataByID(PacketSlot, SIPDecoderID, (void**)&SData)){
		printf("Failed to get SIP Not Version data\n");
		return FALSE;
	}

	Version=SData->sip_version;
	
#ifdef DEBUGMATCH
	printf("**************************************\n");
	printf("Before applying sip not version tests\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	

	
    Node=Nodes;
   	while(Node){
       	if (RuleIsActive(PacketSlot,Node->RuleID)){
	if (strlen(Version) < 10) {
	for (i=0;i<12;i++)tVersion[i]='\0';
	tVersion[0]=',';

	for (i=0;i<strlen(Version);i++) tVersion[i+1]=Version[i];
	tVersion[strlen(Version)+1]=',';

#ifdef DEBUG	
	printf("%s\n",tVersion);
	printf("%s\n",ListVersion);
	printf("%i\n",strlen(tVersion));
	printf("%i\n",strlen(ListVersion));
#endif
	if(strstr(ListVersion,tVersion)==NULL) {
#ifdef DEBUG	
	printf("DISPARAR\n");
#endif

	}else {
#ifdef DEBUG	
	printf("NAO DISPARAR\n");
#endif
#ifdef DEBUGMATCH
           	    printf("SIP Not Version doesn't match, Rule = %i\n",Node->RuleID);
#endif
               	SetRuleInactive(PacketSlot,Node->RuleID);
	}
       	} else {
#ifdef DEBUG	
printf("VERSION = %s\n", Version);
#endif
	}
	}
       	Node=Node->Next;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif
		
	return TRUE;
}

/******************************************
* Add a rule node to this test
******************************************/
int SIPNotVersionAddNode(int TestID, int RuleID, char* Args){
	SIPVersion*			data;
	int					i;
	char*				version;
	char*				m;
	char*				tmp;
	char				listversion[1024];
	char				tArgs[1024];

#ifdef DEBUGPATH
	printf("In SIPVersionAddNode\n");
#endif

#ifdef DEBUG
	printf("Adding a Node with args %s\n",Args);
#endif

	for (i=0;i<1024;i++) listversion[i]='\0';

	listversion[0]=',';
	for (i=0;i<strlen(Args);i++) listversion[i+1]=Args[i];
	listversion[strlen(Args)+1]=',';
//	printf("---->%s\n",listversion);
	strcpy(ListVersion,listversion);
//	printf("---->%s\n",ListVersion);
	version=strtok(Args,",");
	tmp=version;
	while (version!=NULL) {
		version=strtok(NULL,",");
		m=version;
		if (version==NULL) m=tmp;
#ifdef DEBUG
		printf("VERSION=%s\n",m);
#endif
		data=NULL;
		data=calloc(sizeof(SIPVersion),1);
		data->Version=m;
	
		/*check to see if this is a duplicate*/
#ifdef DEBUG
			printf("First SIP Version %s\n", data->Version);
#endif	
			TestAddNode(TestID, RuleID, (void*)data);
		data=NULL;
		free(data);
	}

	return TRUE;
}

/****************************************
* Set up the test of the SIP Not Version Field
*****************************************/
int InitTestSIPNotVersion(){
	int	TestID;

#ifdef DEBUGPATH
	printf("In InitTestSIPNotVersion\n");
#endif

	SIPVersionHead=NULL;

	TestID=CreateTest("SIPNotVersion");
	if (TestID==TEST_NONE) return FALSE;
	
	if (!BindTestToDecoder(TestID, "SIP")){
		printf("Failed to Bind to SIP\n");
		return FALSE;
	} 
	
	snprintf(Globals.Tests[TestID].ShortName, MAX_NAME_LEN, "not_version");
	Globals.Tests[TestID].AddNode=SIPNotVersionAddNode;
	Globals.Tests[TestID].TestFunc=TestSIPNotVersion;
	
	SIPDecoderID=GetDecoderByName("SIP");

	return TRUE;
}
