#include "test_sip_register_flood.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include "../decoders/decode_sip.h"
#include "../decoders/decode_ip.h"
#include "../decoders/decode_udp.h"
#include "../decoders/decode_tcp.h"
#include "../packets/packet.h"
#include <arpa/inet.h>
#include "../engine/num_list.h"
#include "../engine/bits.h"
#include <osipparser2/osip_parser.h>

extern GlobalVars	Globals;


typedef struct my_sip_register_flood_data{
	char* username;
	unsigned short							source;
	unsigned short							dest;
	unsigned int							saddr;
	unsigned int							daddr;
	int										protocol;
	struct timeval							current_time;
	struct timeval							last_time;
	struct timeval							first_time;
	int								state;
	int								limit;
	int								release;
	unsigned int							step;
	struct my_sip_register_flood_data*		Next;
} SIPRegisterFloodData;

//#define DEBUG
//#define DEBUGMATCH
//#define DEBUGPATH

int IPDecoderID;
int SIPDecoderID;
int UDPDecoderID;
int TCPDecoderID;
SIPRegisterFloodData* SIPRegisterFloodDataHead;
int SIPsize;
float coefficient;
int LIMIT;
int RELEASE;
int slice;
void quicksort(double x[], int first, int last);

int RegisterFloodTimerFunc(int TimerID, int Time, void* User){
	SIPRegisterFloodData*			t;
	SIPRegisterFloodData*			last;
	int i;

#ifdef DEBUGPATH
	printf("In RegisterFloodTimerFunc\n");
#endif
	t=SIPRegisterFloodDataHead;
	last=t;
	while(t){
	//	printf("%s %i",t->username, t->release);
		if (t->limit>0) { t->limit=LIMIT; }
		if (t->release < RELEASE) {
		t->release++;
		}
		if (t->release==RELEASE) {
			if (last==t) {
				if (t->Next!=NULL){
					SIPRegisterFloodDataHead=t->Next;
					t=SIPRegisterFloodDataHead;
					last=t;
				} else {
					SIPRegisterFloodDataHead=NULL;
					t=NULL;
				}
			} else if (t->Next==NULL) {
				last->Next=NULL;
				t=NULL;
			} else {
				last->Next=t->Next;
				t=t->Next;
			}
		}
		last=t;
		t=t->Next;
	}
	printf("DEFAULT MESSAGE: REGISTER Flood Timer\n");
}

/******************************************
* Apply the Test with collapsed rules
******************************************/
int TestSIPRegisterFlood(int PacketSlot, TestNode* Nodes){
	int			 					Action;
	double							ttime;
	double							threshold;
	double							thresholdup;
	double							thresholddown;
	double							NormalizedTime;
	double*					TimeDifference;
	SIPRegisterFloodData*			data;
	SIPRegisterFloodData*			t;
	SIPRegisterFloodData*			last;
	SIPRegisterFloodData*			temp;
	TestNode*						Node;
	osip_message_t*					SData;
	IPData*							IData;
	UDPData*						UData;
	TCPData*						TData;
	int								i;
	int								j;
	int					q1;
	int					q3;
	int					qd;
	double					quad1;
	double					quad3;
	PacketRec*						p;

#ifdef DEBUGPATH
	printf("In TestSIPRegisterFlood\n");
#endif

#ifdef DEBUG
	printf("Testing SIP Register\n");
#endif	
	
	IData=NULL;
	SData=NULL;
	UData=NULL;
	TData=NULL;

	if (!Nodes) return FALSE;
	
	p=&Globals.Packets[PacketSlot];
	
	/*get the sip method of the packet*/
	if (!GetDataByID(PacketSlot, IPDecoderID, (void**)&IData)){
#ifdef DEBUG
		printf("Failed to get SIP Invite Flood IP data\n");
#endif	
	}
	if (!GetDataByID(PacketSlot, UDPDecoderID, (void**)&UData)){
		if (!GetDataByID(PacketSlot, TCPDecoderID, (void**)&TData)){
#ifdef DEBUG
			printf("Failed to get SIP Invite Flood UDP data\n");
#endif	
		}
	}
//	if (!GetDataByID(PacketSlot, SIPDecoderID, (void**)&SData)){
	SData=SIPParser2((char*)(p->RawPacket+p->BeginData),SData);
	if (SData==NULL){	
		osip_message_free(SData);
#ifdef DEBUG
		printf("Failed to get SIP Method data\n");
#endif 
		return FALSE;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	printf("Before applying sip register flood tests\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	

    Node=Nodes;
	while(Node){
	       	if (RuleIsActive(PacketSlot,Node->RuleID)){
			if (SData){
#ifdef DEBUG
				printf("         SIP DATA\n");
#endif	
				Action=0;
				if ((SData->sip_method!=NULL)&&(strcmp(SData->sip_method,"REGISTER")==0)) {
					Action = 1;
#ifdef DEBUG
				printf("FROM %s\n", SData->from->url->password);
				printf("TO   %s\n", SData->to->url->password);
#endif	
				}
				if (SData->status_code == 401) Action = 2;
				if (SData->status_code == 200) Action = 3;
				if (SData->status_code == 403) Action = 4;
				if (Action!=0) {
#ifdef DEBUG
					printf("         	REGISTER FLOOD DATA %i\n", Action);
#endif	
					data=calloc(sizeof(SIPRegisterFloodData),1);
					data->saddr=IData->Header->saddr;
					data->daddr=IData->Header->daddr;
					data->protocol=0;
					if (UData){
						data->source=UData->Header->source;
						data->dest=UData->Header->dest;
						data->protocol=1;
					} else if (TData) {
						data->source=TData->Header->source;
						data->dest=TData->Header->dest;
						data->protocol=2;
					}
			                                        char *t1 = SData->from->url->username;
                        			                char *t2;
			                                        if (data->username = malloc(strlen(SData->from->url->username) + 1)) {
                        			                        t2 = data->username;
			                                                while ((*t2++ = *t1++));
								}
					if (data->protocol!=0){
						if (!SIPRegisterFloodDataHead) {
							if (Action==1) {
#ifdef DEBUG
								printf(">>>> First Register Flood Data\n");
#endif	
								gettimeofday(&data->current_time,0);
								data->state=100;
								data->step=0;
								data->last_time=data->current_time;
								data->first_time=data->current_time;
								data->limit=LIMIT;
								data->release=RELEASE+1;


			                                        /*char *t1 = SData->from->url->username;
                        			                char *t2;
			                                        if (data->username = malloc(strlen(SData->from->url->username) + 1)) {
                        			                        t2 = data->username;
			                                                while ((*t2++ = *t1++));
								}*/





								SIPRegisterFloodDataHead=data;
								SIPsize=1;
#ifdef DEBUG
								printf("SRC %u\n",ntohs(data->source));
								printf("DST %u\n",ntohs(data->dest));
								printf("IP %s->",inet_ntoa(*(struct in_addr*)&data->saddr));
								printf("%s\n",inet_ntoa(*(struct in_addr*)&data->daddr));
								printf("PROTO %i\n",data->protocol);
								printf("STATE %i\n",data->state);
#endif
							}
      							SetRuleInactive(PacketSlot,Node->RuleID);
						} else {
							t=SIPRegisterFloodDataHead;
							last=t;
							i=0;
							while(t){
#ifdef DEBUG
								printf(">> DATA VALUES\n");
								printf("SRC %u\n",ntohs(data->source));
								printf("DST %u\n",ntohs(data->dest));
								printf("IP %s->",inet_ntoa(*(struct in_addr*)&data->saddr));
								printf("%s\n",inet_ntoa(*(struct in_addr*)&data->daddr));
								printf("PROTO %i\n",data->protocol);
								printf("STATE %i\n",data->state);
								printf(">> T VALUES\n");
								printf("SRC %u\n",ntohs(t->source));
								printf("DST %u\n",ntohs(t->dest));
								printf("IP %s->",inet_ntoa(*(struct in_addr*)&t->saddr));
								printf("%s\n",inet_ntoa(*(struct in_addr*)&t->daddr));
								printf("PROTO %i\n",t->protocol);
								printf("STATE %i\n",t->state);
#endif	
								if (((data->saddr==t->saddr)&&(data->daddr==t->daddr)&&(data->source==t->source)&&(data->dest==t->dest)&&(data->protocol==t->protocol)&&(strcmp(data->username,t->username)==0))||((data->saddr==t->daddr)&&(data->daddr==t->saddr)&&(data->source==t->dest)&&(data->dest==t->source)&&(data->protocol==t->protocol))&&(strcmp(data->username,t->username)==0)){
									i=i+1;
									if (t->limit>0) {
#ifdef DEBUG
									printf(">>>> Adding Register Flood Data\n");
									printf("SRC %u\n",ntohs(t->source));
									printf("DST %u\n",ntohs(t->dest));
									printf("IP %s->",inet_ntoa(*(struct in_addr*)&t->saddr));
									printf("%s\n",inet_ntoa(*(struct in_addr*)&t->daddr));
									printf("PROTO %i\n",t->protocol);
									printf("STATE %i\n",t->state);
#endif
									switch (t->state) {
									case 100:
									case 200:
										switch (Action) {
										case 1:
											if ((t->state)==100)
												t->state=200;
										break;
										case 2:
											t->state=500;
       			       								SetRuleInactive(PacketSlot,Node->RuleID);
											break;
										case 3:
										case 4:
											t->state=300;
       			       								SetRuleInactive(PacketSlot,Node->RuleID);
										break;
										default:
											//printf("LIMIT = %i\n", t->limit);
											t->limit--;
											if (t->limit>0) {
       				       							SetRuleInactive(PacketSlot,Node->RuleID);
											}
										}
									break;
									case 300:
										if (Action==1) t->state=100;
       		       								SetRuleInactive(PacketSlot,Node->RuleID);
									break;
									case 400:
										switch (Action) {
										case 1:
											t->state=100;
       			       								SetRuleInactive(PacketSlot,Node->RuleID);
										break;
										case 3:
										case 4:
											t->state=300;
       			       								SetRuleInactive(PacketSlot,Node->RuleID);
										break;
										default:
											//printf("LIMIT = %i\n", t->limit);
											t->limit--;
											if (t->limit>0) {
       				       							SetRuleInactive(PacketSlot,Node->RuleID);
											}
										}
									break;
									case 500:
										if (Action==1) t->state=400;
       		       								SetRuleInactive(PacketSlot,Node->RuleID);
									break;
									default:
										if ((NormalizedTime>thresholddown)&&(NormalizedTime<thresholdup)){
       					       						SetRuleInactive(PacketSlot,Node->RuleID);
										} else {
											//printf("LIMIT = %i\n", t->limit);
											t->limit--;
											if (t->limit>0) {
       				       								SetRuleInactive(PacketSlot,Node->RuleID);
											}
										}
									}
									if ((t->state==100)||(t->state==200)){
#ifdef DEBUG
										printf("------RUNNING ATI-----\n");
#endif	
										t->step=t->step+1;
										threshold=(double)1/t->step;
										thresholdup=threshold+(threshold/2);
										thresholddown=threshold-(threshold/2);
										t->last_time=t->current_time;
										gettimeofday(&t->current_time,0);
										ttime=(unsigned)t->current_time.tv_usec-(unsigned)t->last_time.tv_usec;
										NormalizedTime=(double)ttime/((unsigned)t->current_time.tv_usec-(unsigned)t->first_time.tv_usec);

										if ((NormalizedTime>thresholddown)&&(NormalizedTime<thresholdup)){
											if (SIPsize>4){
#ifdef DEBUG
												printf("------RUNNING ASD-----\n");
#endif	
												TimeDifference=calloc(sizeof(double)*SIPsize,1);
												temp=SIPRegisterFloodDataHead;
												j=0;
												while(temp){
													TimeDifference[j++]=(double)((unsigned)temp->current_time.tv_usec-(unsigned)temp->last_time.tv_usec);
													temp=temp->Next;
												}
												quicksort(TimeDifference,0,sizeof(TimeDifference)/sizeof(double));
												qd=((int)SIPsize/4);
												q1=qd+1;
												q3=SIPsize-qd;
												if (SIPsize%2==1){
													quad1=(TimeDifference[q1]+TimeDifference[q1+1])/2;
													quad3=(TimeDifference[q3]+TimeDifference[q3-1])/2;
												} else {
													quad1=TimeDifference[q1];
													quad3=TimeDifference[q3];
												}
												free(TimeDifference);
												if ((ttime>(quad1-quad1*coefficient))&&(ttime<(quad3+quad3*coefficient))) {
       					       								SetRuleInactive(PacketSlot,Node->RuleID);
												} else {
													//printf("LIMIT = %i\n", t->limit);
													t->limit--;
													if (t->limit>0) {
       				       										SetRuleInactive(PacketSlot,Node->RuleID);
													}
												}
											} else {
       				       								SetRuleInactive(PacketSlot,Node->RuleID);
											}
										} else {
											//printf("LIMIT = %i\n", t->limit);
											t->limit--;
											if (t->limit>0) {
       				       								SetRuleInactive(PacketSlot,Node->RuleID);
											}
										}
									}
									}else{
									//	printf("ZERO\n");
										t->release=0;
									}
								}
								last=t;
								t=t->Next;
							}
							if (i==0) { 
								if (Action==1) {
#ifdef DEBUG
									printf(">>>> This is a new one\n");
#endif

									gettimeofday(&data->current_time,0);
									data->state=100;
									data->step=0;
									data->last_time=data->current_time;
									data->first_time=data->current_time;
									data->limit=LIMIT;
									data->release=RELEASE+1;
			                                        	/*char *t1 = SData->from->url->username;
	                        			                char *t2;
				                                        if (data->username = malloc(strlen(SData->from->url->username) + 1)) {
                	        			                        t2 = data->username;
				                                                while ((*t2++ = *t1++));
									}*/
									last->Next=data;
									SIPsize=SIPsize+1;
#ifdef DEBUG
									printf("SRC %u\n",ntohs(data->source));
									printf("DST %u\n",ntohs(data->dest));
									printf("IP %s->",inet_ntoa(*(struct in_addr*)&data->saddr));
									printf("%s\n",inet_ntoa(*(struct in_addr*)&data->daddr));
									printf("PROTO %i\n",data->protocol);
									printf("STATE %i\n",data->state);
#endif	
								}
      								SetRuleInactive(PacketSlot,Node->RuleID);
							}
						}
					} else {
   	       					SetRuleInactive(PacketSlot,Node->RuleID);
					}
				} else {
   		       			SetRuleInactive(PacketSlot,Node->RuleID);
				}
			}else{
       				SetRuleInactive(PacketSlot,Node->RuleID);
			}
		}
       		Node=Node->Next;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif
	osip_message_free(SData);
	return TRUE;
}

/******************************************
* Add a rule node to this test
******************************************/
int SIPRegisterFloodAddNode(int TestID, int RuleID, char* Args){
	SIPRegisterFloodData*			data;

#ifdef DEBUGPATH
	printf("In SIPRegisterFloodAddNode\n");
#endif

	sscanf(Args, "%f, %i, %i, %i", &coefficient, &LIMIT, &slice, &RELEASE);
//	printf("RELEASE = %i LIMIT=%i", RELEASE, LIMIT);
	data=calloc(sizeof(SIPRegisterFloodData),1);
	
	CreateTimer("Register Flodding", slice, RegisterFloodTimerFunc, (void*)NULL);	
	TestAddNode(TestID, RuleID, (void*)data);
	

	SIPsize=0;
	data=NULL;
	free(data);

	return TRUE;
}

/****************************************
* Set up the test of the SIP Register Flood Field
*****************************************/
int InitTestSIPRegisterFlood(){
	int	TestID;

#ifdef DEBUGPATH
	printf("In InitTestSIPRegisterFlood\n");
#endif

	SIPRegisterFloodDataHead=NULL;

	TestID=CreateTest("SIPRegisterFlood");
	if (TestID==TEST_NONE) return FALSE;
	
	if (!BindTestToDecoder(TestID, "SIP")){
#ifdef DEBUG
		printf("Failed to Bind to SIP\n");
#endif
		return FALSE;
	} 
	
	snprintf(Globals.Tests[TestID].ShortName, MAX_NAME_LEN, "register_flood");
	Globals.Tests[TestID].AddNode=SIPRegisterFloodAddNode;
	Globals.Tests[TestID].TestFunc=TestSIPRegisterFlood;
	
	IPDecoderID=GetDecoderByName("IP");
	SIPDecoderID=GetDecoderByName("SIP");
	UDPDecoderID=GetDecoderByName("UDP");
	TCPDecoderID=GetDecoderByName("TCP");

	return TRUE;
}

void mqsort(double x[], int first, int last) {
     int pivIndex = 0;
     if(first < last) {
         pivIndex = partition(x,first, last);
         mqsort(x,first,(pivIndex-1));
         mqsort(x,(pivIndex+1),last);
     }
}

void quicksort(double x[], int first, int last) {
	mqsort(x,first,last);
	int i;
	for (i=1;i<=last;i++)
		x[i-1]=x[i];
}
 
int partition(double y[], int f, int l) {
     int up,down;
     double temp;
     double piv = y[f];
     up = f;
     down = l;
     goto partLS;
     do { 
         temp = y[up];
         y[up] = y[down];
         y[down] = temp;
     partLS:
         while (y[up] <= piv && up < l) {
             up++;
         }
         while (y[down] > piv  && down > f ) {
             down--;
         }
     } while (down > up);
     y[f] = y[down];
     y[down] = piv;
     return down;
 }

