#include "test_siprtp_bye_attack.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../decoders/decode_sip_rtp.h"
#include "../decoders/decode_udp.h"
#include "../decoders/decode_ip.h"
#include "../packets/packet.h"
#include <arpa/inet.h>
#include "../engine/num_list.h"
#include "../engine/bits.h"
#include <osipparser2/osip_parser.h>

extern GlobalVars	Globals;

typedef struct my_bye_attack_data{
	unsigned short					source;
	unsigned short					dest;
	unsigned int					saddr;
	unsigned int					daddr;
	int								count;
	int								ttl;
	struct my_bye_attack_data*		Next;
} ByeAttackData;

typedef struct my_lookup_data{
	unsigned int					saddr;
	unsigned int					daddr;
	struct my_lookup_data*			Next;
} LookupData;

//#define DEBUG
//#define DEBUGMATCH
//#define DEBUGPATH

int TTLByeAttack;
int ThresholdByeAttack;
int MaxSipBound;
int SIPRTPDecoderID;
int IPDecoderID;
int UDPDecoderID;
ByeAttackData* ByeAttackDataHead;
LookupData* LookupDataHead;

int LookupCount(unsigned int saddr, unsigned int daddr, unsigned short source, unsigned short dest){
	ByeAttackData*		t;
	ByeAttackData*		last;

	if (saddr==0) return 0;
	if (daddr==0) return 0;

	t=ByeAttackDataHead;
	last=t;
	while(t){
		if ((t->saddr==saddr)&&(t->daddr==daddr)&&(t->source==source)&&(t->dest==dest)){
			return t->count;
		}
		last=t;
		t=t->Next;
	}
	return 0;
}

/******************************************
* Apply the Test with collapsed rules
******************************************/
int TestSIPRTPByeAttack(int PacketSlot, TestNode* Nodes){
	int					difference;
	ByeAttackData*		data;
	ByeAttackData*		t;
	ByeAttackData*		last;
	LookupData*			dataBye;
	LookupData*			tBye;
	LookupData*			lastBye;
	TestNode*			Node;
	osip_message_t*		SData;
	IPData*				IData;
	RTPData*			RData;
	UDPData*			UData;
	int					i;
	PacketRec*			p;

#ifdef DEBUGPATH
	printf("In TestSIPRTPByeAttack\n");
#endif

#ifdef DEBUG
	printf("Testing SIP RTP Bye Attack\n");
#endif	
	
	IData=NULL;
	SData=NULL;
	RData=NULL;
	UData=NULL;

	if (!Nodes) return FALSE;
	
	p=&Globals.Packets[PacketSlot];
	
	/*get the sip method of the packet*/
	if (!GetDataByID(PacketSlot, IPDecoderID, (void**)&IData)){
#ifdef DEBUG
		printf("Failed to get SIP RTP Bye Attack IP data\n");
#endif	
	}
	if (!GetDataByID(PacketSlot, UDPDecoderID, (void**)&UData)){
#ifdef DEBUG
		printf("Failed to get SIP RTP Bye Attack UDP data\n");
#endif	
	}

	if (GetDataByID(PacketSlot, SIPRTPDecoderID, (void**)&RData)){
		if (!RTPHeaderVerification(RData)) {
#ifdef DEBUG
			printf("Failed to get SIP RTP Bye Attack RTP data\n");
#endif	
			RData=NULL;
			if (GetDataByID(PacketSlot, SIPRTPDecoderID, (void**)&SData)){
				SData=SIPParser2((char*)(p->RawPacket+p->BeginData),SData);
				if (SData==NULL) {
#ifdef DEBUG
					printf("Failed to get SIP RTP Bye Attack SIP data\n");
#endif	
				}
			}
		}
	}


#ifdef DEBUGMATCH
	printf("**************************************\n");
	printf("Before applying sip rtp bye tests\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	



    Node=Nodes;
   	while(Node){
       	if (RuleIsActive(PacketSlot,Node->RuleID)){
			if (SData){
#ifdef DEBUG
				printf("         SIP DATA\n");
#endif	
      			SetRuleInactive(PacketSlot,Node->RuleID);
				if (SData->sip_method!=NULL){
					if (strcmp(SData->sip_method,"BYE")==0){
#ifdef DEBUG
						printf("SIP DATA BYE\n");
#endif	
						dataBye=calloc(sizeof(LookupData),1);
						dataBye->saddr=IData->Header->saddr;
						dataBye->daddr=IData->Header->daddr;
#ifdef DEBUG
						printf("SRC_BYE %s\n",inet_ntoa(*(struct in_addr*)&dataBye->saddr));
						printf("DST_BYE %s\n",inet_ntoa(*(struct in_addr*)&dataBye->daddr));
#endif
						if (!LookupDataHead) {
#ifdef DEBUG
							printf("*****************************First Bye Data\n");
#endif	
							LookupDataHead = dataBye;
						} else {
							tBye=LookupDataHead;
							lastBye=tBye;
							i=0;
							while(tBye){
								if(((tBye->saddr==dataBye->saddr)&&(tBye->daddr==dataBye->daddr))||((tBye->daddr==dataBye->saddr)&&(tBye->saddr==dataBye->daddr))){
									i+=1;
									break;
								}
								lastBye=tBye;
								tBye=tBye->Next;
							}
							if(i==0){
#ifdef DEBUG
								printf("***********************************New Bye Data\n");
#endif	
								lastBye->Next=dataBye;
							}
						}
						dataBye=NULL;
						free(dataBye);
					}
				}
			} else if ((IData)&&(!SData)&&(RData)&&(UData)){
				if (RTPHeaderVerification(RData)){
#ifdef DEBUG
					printf("         RTP DATA\n");
					printf("SRC_BYE %s\n",inet_ntoa(*(struct in_addr*)&dataBye->saddr));
					printf("DST_BYE %s\n",inet_ntoa(*(struct in_addr*)&dataBye->daddr));
    				printf("USRC %u\n", ntohs(UData->Header->source));
    				printf("UDST %u\n", ntohs(UData->Header->dest));
#endif
					data=calloc(sizeof(ByeAttackData),1);
					data->saddr=IData->Header->saddr;
					data->daddr=IData->Header->daddr;
					data->source=UData->Header->source;
					data->dest=UData->Header->dest;

					if (!ByeAttackDataHead) {
#ifdef DEBUG
						printf("First Bye Attack Data\n");
#endif	
						data->count=0;
						data->ttl=0;
#ifdef DEBUG
						printf("COUNT=%i\n",data->count);
#endif	
						ByeAttackDataHead=data;
					} else {
#ifdef DEBUG
						printf("Adding Bye Attack Data\n");
#endif	
						t=ByeAttackDataHead;
						last=t;
						i=0;
						while(t){
							if ((data->saddr==t->saddr)&&(data->daddr==t->daddr)&&(data->source==t->source)&&(data->dest==t->dest)){
								i=i+1;
								t->ttl=0;
								t->count=t->count + 1;
							}else{
								t->ttl=t->ttl+1;
							}
							if (t->ttl>TTLByeAttack){
#ifdef DEBUG
								printf("--------------TTL over\n");
#endif	
								if (last==t) {
									if (t->Next!=NULL){
										ByeAttackDataHead=t->Next;
										t=ByeAttackDataHead;
										last=t;
									} else {
										ByeAttackDataHead=NULL;
										t=NULL;
									}
								} else if (t->Next==NULL) {
									last->Next=NULL;
									t=NULL;
								} else {
									last->Next=t->Next;
									t=t->Next;
								}
							} else {
								last=t;
								t=t->Next;
							}
						}
						if (i==0) { 
#ifdef DEBUG
							printf("This is a new one\n");
#endif		
							data->count=0;
							data->ttl=0;
							last->Next=data;
						}
					}
					data=NULL;
					free(data);
					if(LookupDataHead){
#ifdef DEBUG
						printf("Looking to match\n");
#endif		
						tBye=LookupDataHead;
						lastBye=tBye;
						while(tBye){
							if (!tBye->saddr) tBye->saddr=0;
							if (!tBye->daddr) tBye->daddr=0;
							difference = LookupCount(tBye->saddr,tBye->daddr,UData->Header->source,UData->Header->dest)-LookupCount(tBye->daddr,tBye->saddr,UData->Header->dest,UData->Header->source);

							if (difference<0) difference*=-1;
#ifdef DEBUG
							printf("DIFFERENCE === %i\n",difference);
#endif
							if (difference<ThresholdByeAttack){
#ifdef DEBUGMATCH
		        		   	    printf("SIP Bye Attack doesn't match, Rule = %i\n",Node->RuleID);
#endif
        		       			SetRuleInactive(PacketSlot,Node->RuleID);
							}
							lastBye=tBye;
							tBye=tBye->Next;
						}
					} else {
   		       			SetRuleInactive(PacketSlot,Node->RuleID);
					}
				}else{
       				SetRuleInactive(PacketSlot,Node->RuleID);
				}
			}else{
       			SetRuleInactive(PacketSlot,Node->RuleID);
			}
		}
       	Node=Node->Next;
	}

#ifdef DEBUGMATCH
	printf("**************************************\n");
	for (i=0;i<Globals.NumRules;i++)
	if (RuleIsActive(PacketSlot,i))
		printf("Rule %i is active\n",i);
	else
		printf("Rule %i is inactive\n",i);
	printf("**************************************\n");
#endif	
	return TRUE;
}

/******************************************
* Add a rule node to this test
******************************************/
int SIPRTPByeAttackAddNode(int TestID, int RuleID, char* Args){
	RTPData*			data;

#ifdef DEBUGPATH
	printf("In SIPRTPByeAttackAddNode\n");
#endif

#ifdef DEBUG
	printf("Adding a Node with args %s\n",Args);
#endif

	ThresholdByeAttack = atoi(Args);
	TTLByeAttack = ThresholdByeAttack * 2;
	data=NULL;
	data=calloc(sizeof(RTPData),1);
	
#ifdef DEBUG
	printf("First SIP RTP Bye Attack\n");
#endif	
	TestAddNode(TestID, RuleID, (void*)data);

	data=NULL;
	free(data);
	return TRUE;
}

/****************************************
* Set up the test of the SIP RTP Bye Attack Field
*****************************************/
int InitTestSIPRTPByeAttack(){
	int	TestID;

#ifdef DEBUGPATH
	printf("In InitTestSIPRTPByeAttack\n");
#endif

	ByeAttackDataHead=NULL;
	LookupDataHead=NULL;

	TestID=CreateTest("SIPRTPByeAttack");
	if (TestID==TEST_NONE) return FALSE;
	
	if (!BindTestToDecoder(TestID, "SIPRTP")){
		printf("Failed to Bind to SIPRTP\n");
		return FALSE;
	} 
	
	snprintf(Globals.Tests[TestID].ShortName, MAX_NAME_LEN, "bye_attack");
	Globals.Tests[TestID].AddNode=SIPRTPByeAttackAddNode;
	Globals.Tests[TestID].TestFunc=TestSIPRTPByeAttack;
	
	SIPRTPDecoderID=GetDecoderByName("SIPRTP");
	IPDecoderID=GetDecoderByName("IP");
	UDPDecoderID=GetDecoderByName("UDP");

	return TRUE;
}
